/* 
 * Core Interface for F-Prot Antivirus Daemon			
 *
 * Copyright (C) Rainer Link, 2001-2002
 *               OpenAntiVirus.org <rainer@openantivirus.org>
 *
 * Credits to W. Richard Stevens - RIP
 * 
 * This software is licensed under the GNU General Public License (GPL)
 * See: http://www.gnu.org/copyleft/gpl.html
 *
*/

#include <includes.h>
#include "vscan-fprotd_core.h"
 

/* print a message via syslog */
void vscan_syslog(char *printMessage, ...)
{
        char printMsg[512];

        va_list argptr;
        va_start(argptr, printMessage);
        vsnprintf(printMsg, sizeof(printMsg)-1, printMessage, argptr);
        va_end(argptr);

	syslog(SYSLOG_PRIORITY, "%s", printMsg);
}



/* initialise socket to F-Prot Daemon 
   returns -1 on error or the socket descriptor */
int vscan_fprotd_init(void)
{

	int sockfd;
	struct sockaddr_in servaddr;

	/* create socket */
        if (( sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0 ) {
               vscan_syslog("ERROR: can not create socket!");
               return -1; 
        }

	bzero(&servaddr, sizeof(servaddr));
        servaddr.sin_family = AF_INET;
        servaddr.sin_port = htons(FPROTD_PORT);

	/* hm, inet_pton may not exist on all systems - FIXME ! */
        if ( inet_pton(AF_INET, FPROTD_IP, &servaddr.sin_addr) <= 0 ) {
                vscan_syslog("ERROR: inet_pton failed!");
                return -1;
	}

	/* connect to socket */
	if ( connect(sockfd, (struct sockaddr *) &servaddr, sizeof(servaddr)) < 0 )
        {
                vscan_syslog("ERROR: can not connect to F-Prot Daemon!");
                return -1;
        }


	return sockfd;

}

/*
void vscan_fprotd_syslog(char *syslogMessage, ...)
{
        char logMsg[512];

        va_list argptr;
        va_start(argptr, syslogMessage);
        vsnprintf(logMsg, sizeof(logMsg)-1, syslogMessage, argptr);
        va_end(argptr);

        syslog(SYSLOG_PRIORITY, "%s\n", logMsg);
}

*/

/*
  If virus is found, logs the filename/virusname into syslog
*/
void vscan_fprotd_log_virus(char *infected_file, char *result)
{
	char *str;

	/* remove "<name>" and "</name>"from the result string to get only the virus name - hack alert ;) */
	str = result;
	str+= 6;
	str[strlen(str)-8] = '\0';

        vscan_syslog("Scan result: '%s' infected with virus '%s'", infected_file, str);
        
}



/*
  Scans a file (*FILE*, not a directory - keep that in mind) for a virus
  Expects socket descriptor and file name to scan for
  Return -1 on error, 0 if no virus was found, 1 if a virus was found 
*/
int vscan_fprotd_scanfile(int sockfd, char *scan_file)
{
	size_t len;
	char recvline[MAXLINE + 1];
	pstring fprotdCommand;	/* the command line to be send to daemon */
	char *str;
	FILE *fpin, *fpout;
	bool received_data = FALSE; /* indicates, if any response from deamon was received */

	/* open stream sockets */
        fpin = fdopen(sockfd, "r");
        if ( fpin == NULL ) {
                vscan_syslog("ERROR: Can not open stream for reading - %s", strerror(errno));
                return -1;
        }

        fpout = fdopen(sockfd, "w");
        if ( fpout == NULL ) {
                vscan_syslog("ERROR: Can not open stream for writing - %s", strerror(errno));
                return -1;
        }


        #if VSCAN_FPROTD_VERBOSE_FILE_LOGGING==1
        vscan_syslog("Scanning file : '%s'", scan_file);
        #endif

	/* F-Prot Daemon expects "GET <filename>[?<arguments>] HTTP/1.0\r\n\r\n" */
	/* what about if the <filename> itself contains '\n'? */
        fstrcpy(fprotdCommand, "GET ");
        fstrcat(fprotdCommand, scan_file);
	fstrcat(fprotdCommand, "?");
	fstrcat(fprotdCommand, FPROTD_ARGS);
        fstrcat(fprotdCommand, " HTTP/1.0\r\n\r\n");

	/* write to socket */
	/* NOTE: what happens if scan_file is very long? */
	len = strlen(fprotdCommand);
	if ( len < 0 ) {
		/* overflow? */
		vscan_syslog("ERROR: file name is too long!");
		return -1;
	}	

	if ( fputs(fprotdCommand, fpout) == EOF ) {
		vscan_syslog("ERROR: can not send file name to F-Prot Daemon!");
		return -1;
	}

	/* hum, instead of flush()ing, use setvbuf to set to line-buffering? */
        if ( fflush(fpout) == EOF ) {
                vscan_syslog("ERROR: can not flush output stream - %s", strerror(errno));
        }


	/* read from socket, line by line */
	setvbuf(fpin, (char *)NULL, _IOLBF, 0);
	/* setlinebuf(fpin); */

	while ( (fgets(recvline, MAXLINE, fpin)) != NULL ) {

		received_data = TRUE;

		/* ignore the HTTP response header, remove any leading 
		   white spaces */
		str = NULL;
		str = strchr(recvline, '<');
		if ( str != NULL ) {
			if ( strncmp(str, "<name>", 6) == 0 ) {
				/* virus found */
				vscan_fprotd_log_virus(scan_file, str);
				return 1;
			} else if ( strncmp(str, "<error>", 7) == 0 ) {
				/* ERROR */
				#if VSCAN_FPROTD_VERBOSE_FILE_LOGGING==1
				vscan_syslog("file %s not found or not readable", scan_file);
				#endif
			}
		}
	}

	/* did we receive any data from daemon? */
	if ( !received_data ) {
		vscan_syslog("ERROR: can not get result from F-Prot Daemon
!");
		return -1;
         } else {

	 	/* OK */
		#if VSCAN_FPROTD_VERBOSE_FILE_LOGGING==1
        	vscan_syslog("file %s is clean", scan_file);
        	#endif
	}

	if ( fclose(fpin) == EOF ) {
		vscan_syslog("ERROR: can not close input stream - %s", strerror(errno));
	}

	if ( fclose(fpout) == EOF ) {
		vscan_syslog("ERROR: can not close output stream - %s", strerror(errno));
	}

	return 0;
}


/*
  close socket
*/
void vscan_fprotd_end(int sockfd)
{
	/* sockfd == -1 indicates an error while connecting to socket */
	if ( sockfd >= 0 ) {
		close(sockfd);
	}

}
